@@ -1,3 +1,31 @@
+/****************************************************************************  
+ 
+  Copyright (c) 1999 WU-FTPD Development Group.  
+  All rights reserved.
+  
+  Portions Copyright (c) 1980, 1985, 1988, 1989, 1990, 1991, 1993, 1994
+    The Regents of the University of California.
+  Portions Copyright (c) 1993, 1994 Washington University in Saint Louis.
+  Portions Copyright (c) 1996, 1998 Berkeley Software Design, Inc.
+  Portions Copyright (c) 1989 Massachusetts Institute of Technology.
+  Portions Copyright (c) 1998 Sendmail, Inc.
+  Portions Copyright (c) 1983, 1995, 1996, 1997 Eric P.  Allman.
+  Portions Copyright (c) 1997 by Stan Barber.
+  Portions Copyright (c) 1997 by Kent Landfield.
+  Portions Copyright (c) 1991, 1992, 1993, 1994, 1995, 1996, 1997
+    Free Software Foundation, Inc.  
+ 
+  Use and distribution of this software and its source code are governed 
+  by the terms and conditions of the WU-FTPD Software License ("LICENSE").
+ 
+  If you did not receive a copy of the license, it may be obtained online
+  at http://www.wu-ftpd.org/license.html.
+ 
+  $Id: sco.c,v 1.6 1999/08/27 14:18:42 wuftpd Exp $
+ 
+****************************************************************************/
+#ifndef _SCO_DS			/* none of this is required on SCO OpenServer 5 */
+
 /* Written in 1992, 1993 by Eduard Vopicka, Prague University of Economics */
 
 #include "../src/config.h"
@@ -6,7 +34,7 @@
 #include <stdio.h>
 #include <fcntl.h>
 #include <sys/immu.h>
-#include <sys/dir.h>        /* required by <sys/user.h> */
+#include <sys/dir.h>		/* required by <sys/user.h> */
 #include <sys/user.h>
 #include <sys/signal.h>
 #include <sys/fs/s5param.h>
@@ -20,132 +48,92 @@ static int KmemFd = -1;
 kmem_open()
 {
     if (KmemFd < 0 && (KmemFd = open("/dev/kmem", O_RDWR, 0)) < 0) {
-        syslog(LOG_EMERG, "kmem open failed: %m");
-        exit(1);
+	syslog(LOG_EMERG, "kmem open failed: %m");
+	exit(1);
     }
 }
 
-int
-wrub(addr, off, len)
-    char    *addr;
-    off_t   off;
-    int len;
+int wrub(addr, off, len)
+     char *addr;
+     off_t off;
+     int len;
 {
-    off_t seek_off = UVUBLK+off;
-    kmem_open();                /* make sure it is open */
-    if (lseek(KmemFd, (char*)seek_off, SEEK_SET) != seek_off) {
-        syslog(LOG_EMERG, "lseek failed on /dev/kmem: %m");
-        exit(1);
+    off_t seek_off = UVUBLK + off;
+    kmem_open();		/* make sure it is open */
+    if (lseek(KmemFd, (char *) seek_off, SEEK_SET) != seek_off) {
+	syslog(LOG_EMERG, "lseek failed on /dev/kmem: %m");
+	exit(1);
     }
     if (write(KmemFd, addr, len) != len) {
-        syslog(LOG_EMERG, "write failed on /dev/kmem: %m");
-        exit(1);
+	syslog(LOG_EMERG, "write failed on /dev/kmem: %m");
+	exit(1);
     }
-    return(0);
+    return (0);
 }
 
-void
-s5proctitle(s)
-    char    *s;
-{
-    struct user u;
-    char buf[PSARGSZ];
-    int len = strlen(s);
-    if (len >= PSARGSZ) len = PSARGSZ-1;
-    strncpy(buf,s,len);
-    buf[len] = '\0';
-    wrub(buf, (off_t)u.u_psargs-(off_t)&u, PSARGSZ);
-}
-
-/* #ifndef _M_UNIX     /* before 3.2v4.0 */
 /* UHUHUH, this crazy code is still required for 3.2v4.2 */
 
 uid_t
 seteuid(id)
-    uid_t   id;
+     uid_t id;
 {
-    struct  user u;
-    return(wrub(&id, (off_t)&u.u_uid-(off_t)&u, sizeof(id)));
+    struct user u;
+    return (wrub(&id, (off_t) & u.u_uid - (off_t) & u, sizeof(id)));
 }
 
 uid_t
 setruid(id)
-    uid_t   id;
+     uid_t id;
 {
-    struct  user u;
-    return(wrub(&id, (off_t)&u.u_ruid-(off_t)&u, sizeof(id)));
+    struct user u;
+    return (wrub(&id, (off_t) & u.u_ruid - (off_t) & u, sizeof(id)));
 }
 
 
 uid_t
 setegid(id)
-    uid_t   id;
+     uid_t id;
 {
-    struct  user u;
-    return(wrub(&id, (off_t)&u.u_gid-(off_t)&u, sizeof(id)));
+    struct user u;
+    return (wrub(&id, (off_t) & u.u_gid - (off_t) & u, sizeof(id)));
 }
 
 uid_t
 setrgid(id)
-    uid_t   id;
+     uid_t id;
 {
-    struct  user u;
-    return(wrub(&id, (off_t)&u.u_rgid-(off_t)&u, sizeof(id)));
+    struct user u;
+    return (wrub(&id, (off_t) & u.u_rgid - (off_t) & u, sizeof(id)));
 }
 
 uid_t
 setuid(id)
-    uid_t   id;
+     uid_t id;
 {
-    (void)seteuid(id);
-    (void)setruid(id);
-    return(0);
+    (void) seteuid(id);
+    (void) setruid(id);
+    return (0);
 }
 
 uid_t
 setgid(id)
-    uid_t   id;
+     uid_t id;
 {
-    (void)setegid(id);
-    (void)setrgid(id);
-    return(0);
+    (void) setegid(id);
+    (void) setrgid(id);
+    return (0);
 }
 
 uid_t
 setreuid(ruid, euid)
-    uid_t   ruid, euid;
+     uid_t ruid, euid;
 {
     if (ruid != 0xffff)
-        (void)setruid(ruid);
+	(void) setruid(ruid);
     if (euid != 0xffff)
-        (void)seteuid(euid);
-    return(0);
+	(void) seteuid(euid);
+    return (0);
 }
-/* #endif /* _M_UNIX */
-
-#ifdef SETPROCTITLE
-
-void
-SCOproctitle(va_alist)
-    va_dcl
-{
-    va_list ap;
-    char *fmt;
-
-    register char *p;
-    char buf[BUFSIZ];
-
-    va_start(ap);
-    fmt = va_arg(ap, char *); /* first argument is always the fmt string */
-    p = buf; *p++ = '-'; p += vsprintf(p, fmt, ap);
-    va_end(ap)
-
-    while (*--p == '\r' || *p == '\n' || *p == ' ' || *p == '\t');
-    p = strcpy(++p, " (ftpd)"); /* make ps print our process name */
-/*  while (*p) p++; */
-    s5proctitle(buf);
-}
-#endif /* SETPROCTITLE */
 
 /*
  * Copyright (c) 1989, 1991 The Regents of the University of California.
@@ -194,17 +182,17 @@ SCOproctitle(va_alist)
 #include <strings.h>
 
 #ifndef CONSOLE
-# define CONSOLE "/dev/console"
+#define CONSOLE "/dev/console"
 #endif
 
-static int  LogFile = -1;       /* fd for log */
-static int  LogStat = 0;        /* status bits, set by openlog() */
-static char *LogTag = "syslog"; /* string to tag the entry with */
-static int  LogFacility = LOG_USER; /* default facility code */
-static int  LogMask = 0xff;     /* mask of priorities to be logged */
+static int LogFile = -1;	/* fd for log */
+static int LogStat = 0;		/* status bits, set by openlog() */
+static char *LogTag = "syslog";	/* string to tag the entry with */
+static int LogFacility = LOG_USER;	/* default facility code */
+static int LogMask = 0xff;	/* mask of priorities to be logged */
 
 syslog(va_alist)
-    va_dcl
+     va_dcl
 {
     va_list args;
     int pri;
@@ -221,9 +209,9 @@ syslog(va_alist)
 }
 
 vsyslog(pri, fmt, ap)
-    int pri;
-    register char *fmt;
-    va_list ap;
+     int pri;
+     register char *fmt;
+     va_list ap;
 {
     extern int errno;
     register int cnt;
@@ -244,107 +232,113 @@ vsyslog(pri, fmt, ap)
 #endif
 
     /* see if we should just throw out this message */
-    if ((u_int)LOG_FAC(pri) >= (1 << LOG_NFACILITIES) ||
-        (!(LOG_MASK(LOG_PRI(pri))&LogMask)) ||
-        (pri &~ (LOG_PRIMASK|LOG_FACMASK)))
-        return;
+    if ((u_int) LOG_FAC(pri) >= (1 << LOG_NFACILITIES) ||
+	(!(LOG_MASK(LOG_PRI(pri)) & LogMask)) ||
+	(pri & ~(LOG_PRIMASK | LOG_FACMASK)))
+	return;
     if (LogFile < 0)
-        openlog(LogTag, LogStat | LOG_NDELAY, 0);
+	openlog(LogTag, LogStat | LOG_NDELAY, 0);
 
     /* set default facility if none specified */
     if ((pri & LOG_FACMASK) == 0)
-        pri |= LogFacility;
+	pri |= LogFacility;
 
     /* build the message */
-    (void)time(&now);
-    (void)sprintf(tbuf, "<%d>%.15s ", pri, ctime(&now) + 4); /**/
+    (void) time(&now);
+    (void) sprintf(tbuf, "<%d>%.15s ", pri, ctime(&now) + 4);/**/
 /*  (void)sprintf(tbuf, "%3o %.15s ", pri, ctime(&now) + 4); /**/
     for (p = tbuf; *p; ++p);
 #ifndef LOG_PERROR
 #define LOG_PERROR 0x20
 #endif
     if (LogStat & LOG_PERROR)
-        stdp = p;
+	stdp = p;
     if (LogTag) {
-        (void)strcpy(p, LogTag);
-        for (; *p; ++p);
+	(void) strcpy(p, LogTag);
+	for (; *p; ++p);
     }
     if (LogStat & LOG_PID) {
-        (void)sprintf(p, "[%d]", getpid());
-        for (; *p; ++p);
+	(void) sprintf(p, "[%d]", getpid());
+	for (; *p; ++p);
     }
     if (LogTag) {
-        *p++ = ':';
-        *p++ = ' ';
+	*p++ = ':';
+	*p++ = ' ';
     }
 
     /* substitute error message for %m */
     {
-        register char ch, *t1, *t2;
-/*      char *strerror(); /**/
+	register char ch, *t1, *t2;
+/*      char *strerror(); /* */
 
-        for (t1 = fmt_cpy; ch = *fmt; ++fmt)
-            if (ch == '%' && fmt[1] == 'm') {
-                ++fmt;
-                for (t2 = strerror(saved_errno);
-                    *t1 = *t2++; ++t1);
-            }
-            else
-                *t1++ = ch;
-        *t1 = '\0';
+	for (t1 = fmt_cpy; ch = *fmt; ++fmt)
+	    if (ch == '%' && fmt[1] == 'm') {
+		++fmt;
+		for (t2 = strerror(saved_errno);
+		     *t1 = *t2++; ++t1);
+	    }
+	    else
+		*t1++ = ch;
+	*t1 = '\0';
     }
 
-    (void)vsprintf(p, fmt_cpy, ap);
+    (void) vsprintf(p, fmt_cpy, ap);
 
     cnt = strlen(tbuf);
-    tbuf[cnt++]='\n';
+    tbuf[cnt++] = '\n';
 
     /* output to stderr if requested */
     if (LogStat & LOG_PERROR)
-        write(2, stdp, cnt - (stdp - tbuf));
+	write(2, stdp, cnt - (stdp - tbuf));
 
     /* output the message to the local logger */
     if (write(LogFile, tbuf, cnt) == cnt)
-        return;
+	return;
 
     /* output the message to the console */
     pid = vfork();
     if (pid == -1)
-        return;
+	return;
     if (pid == 0) {
-        int fd;
+	int fd;
+	int saveerrno;
 
-        sigfillset(&newmask); sigdelset(&newmask, SIGALRM);
-        (void) sigprocmask(SIG_SETMASK, &newmask, &oldmask);
-        (void)signal(SIGALRM, SIG_DFL);
-        (void)alarm((u_int)5);
-        if ((fd = open(CONSOLE, O_WRONLY, 0)) < 0)
-            return;
-        (void)alarm((u_int)0);
-        (void)strcat(tbuf, "\r");
-        p = index(tbuf, '>') + 1;
-        (void)write(fd, p, cnt + 1 - (p - tbuf));
-        (void)close(fd);
-        _exit(0);
+	sigfillset(&newmask);
+	sigdelset(&newmask, SIGALRM);
+	(void) sigprocmask(SIG_SETMASK, &newmask, &oldmask);
+	(void) signal(SIGALRM, SIG_DFL);
+	(void) alarm((u_int) 5);
+	fd = open(CONSOLE, O_WRONLY, 0);
+	saveerrno = errno;
+	(void) alarm((u_int) 0);
+	errno = saveerrno;
+	if (fd < 0)
+	    return;
+	(void) strcat(tbuf, "\r");
+	p = index(tbuf, '>') + 1;
+	(void) write(fd, p, cnt + 1 - (p - tbuf));
+	(void) close(fd);
+	_exit(0);
     }
     if (!(LogStat & LOG_NOWAIT))
-        while ((cnt = wait((int *)0)) > 0 && cnt != pid);
+	while ((cnt = wait((int *) 0)) > 0 && cnt != pid);
 }
 
 /*
  * OPENLOG -- open system log
  */
 openlog(ident, logstat, logfac)
-    char *ident;
-    int logstat, logfac;
+     char *ident;
+     int logstat, logfac;
 {
     if (ident != NULL)
-        LogTag = ident;
+	LogTag = ident;
     LogStat = logstat;
-    if (logfac != 0 && (logfac &~ LOG_FACMASK) == 0)
-        LogFacility = logfac;
+    if (logfac != 0 && (logfac & ~LOG_FACMASK) == 0)
+	LogFacility = logfac;
     if (LogFile == -1) {
-        LogFile = open(SYSLOGFILE, O_WRONLY|O_APPEND|O_SYNC);   /**/
+	LogFile = open(SYSLOGFILE, O_WRONLY | O_APPEND | O_SYNC);
+								/**/
 /*      LogFile = open(SYSLOGFILE, O_WRONLY|O_APPEND);      /**/
     }
 }
@@ -362,13 +356,13 @@ closelog()
  * SETLOGMASK -- set the log mask level
  */
 setlogmask(pmask)
-    int pmask;
+     int pmask;
 {
     int omask;
 
     omask = LogMask;
     if (pmask != 0)
-        LogMask = pmask;
+	LogMask = pmask;
     return (omask);
 }
 
@@ -415,8 +409,8 @@ static char sccsid[] = "@(#)getcwd.c    5.11 (Berkeley) 2/24/91";
 
 #include <sys/param.h>
 
-#ifndef HAVE_SYMLINK
-# define lstat stat
+#ifndef HAVE_LSTAT
+#define lstat stat
 #endif
 
 #include <sys/stat.h>
@@ -426,9 +420,9 @@ static char sccsid[] = "@(#)getcwd.c    5.11 (Berkeley) 2/24/91";
 #include <string.h>
 
 #ifdef HAVE_D_NAMLEN
-# define DNAMLEN(dp) (dp->d_namlen)
+#define DNAMLEN(dp) (dp->d_namlen)
 #else
-# define DNAMLEN(dp) (strlen(dp->d_name))
+#define DNAMLEN(dp) (strlen(dp->d_name))
 #endif
 
 #define ISDOT(dp) \
@@ -436,9 +430,9 @@ static char sccsid[] = "@(#)getcwd.c    5.11 (Berkeley) 2/24/91";
         dp->d_name[1] == '.' && dp->d_name[2] == '\0'))
 
 char *
-getcwd(pt, size)
-    char *pt;
-    int size;
+     getcwd(pt, size)
+     char *pt;
+     int size;
 {
     register struct dirent *dp;
     register DIR *dir;
@@ -459,16 +453,17 @@ getcwd(pt, size)
      * is built from the end of the buffer backwards.
      */
     if (pt) {
-        ptsize = 0;
-        if (!size) {
-            errno = EINVAL;
-            return((char *)NULL);
-        }
-        ept = pt + size;
-    } else {
-        if (!(pt = (char *)malloc(ptsize = 1024 - 4)))
-            return((char *)NULL);
-        ept = pt + ptsize;
+	ptsize = 0;
+	if (!size) {
+	    errno = EINVAL;
+	    return ((char *) NULL);
+	}
+	ept = pt + size;
+    }
+    else {
+	if (!(pt = (char *) malloc(ptsize = 1024 - 4)))
+	    return ((char *) NULL);
+	ept = pt + ptsize;
     }
     bpt = ept - 1;
     *bpt = '\0';
@@ -478,8 +473,8 @@ getcwd(pt, size)
      * Should always be enough (it's 340 levels).  If it's not, allocate
      * as necessary.  Special * case the first stat, it's ".", not "..".
      */
-    if (!(up = (char *)malloc(upsize = 1024 - 4)))
-        goto err;
+    if (!(up = (char *) malloc(upsize = 1024 - 4)))
+	goto err;
     eup = up + MAXPATHLEN;
     bup = up;
     up[0] = '.';
@@ -487,141 +482,142 @@ getcwd(pt, size)
 
     /* Save root values, so know when to stop. */
     if (stat("/", &s))
-        goto err;
+	goto err;
     root_dev = s.st_dev;
     root_ino = s.st_ino;
 
-    errno = 0;          /* XXX readdir has no error return. */
+    errno = 0;			/* XXX readdir has no error return. */
 
     for (first = 1;; first = 0) {
-        /* Stat the current level. */
-        if (lstat(up, &s))
-            goto err;
+	/* Stat the current level. */
+	if (lstat(up, &s))
+	    goto err;
 
-        /* Save current node values. */
-        ino = s.st_ino;
-        dev = s.st_dev;
+	/* Save current node values. */
+	ino = s.st_ino;
+	dev = s.st_dev;
 
-        /* Check for reaching root. */
-        if (root_dev == dev && root_ino == ino) {
-            *--bpt = '/';
-            /*
-             * It's unclear that it's a requirement to copy the
-             * path to the beginning of the buffer, but it's always
-             * been that way and stuff would probably break.
-             */
-            (void)bcopy(bpt, pt, ept - bpt);
-            free(up);
-            return(pt);
-        }
+	/* Check for reaching root. */
+	if (root_dev == dev && root_ino == ino) {
+	    *--bpt = '/';
+	    /*
+	     * It's unclear that it's a requirement to copy the
+	     * path to the beginning of the buffer, but it's always
+	     * been that way and stuff would probably break.
+	     */
+	    (void) bcopy(bpt, pt, ept - bpt);
+	    free(up);
+	    return (pt);
+	}
 
-        /*
-         * Build pointer to the parent directory, allocating memory
-         * as necessary.  Max length is 3 for "../", the largest
-         * possible component name, plus a trailing NULL.
-         */
-        if (bup + 3  + MAXNAMLEN + 1 >= eup) {
-            off_t len = bup - up;
-            if (!(ptr = (char *)realloc(up, upsize *= 2)))
-                goto err;
-            up = ptr;
-            bup = up + len;
-            eup = up + upsize;
-        }
-        *bup++ = '.';
-        *bup++ = '.';
-        *bup = '\0';
+	/*
+	 * Build pointer to the parent directory, allocating memory
+	 * as necessary.  Max length is 3 for "../", the largest
+	 * possible component name, plus a trailing NULL.
+	 */
+	if (bup + 3 + MAXNAMLEN + 1 >= eup) {
+	    off_t len = bup - up;
+	    if (!(ptr = (char *) realloc(up, upsize *= 2)))
+		goto err;
+	    up = ptr;
+	    bup = up + len;
+	    eup = up + upsize;
+	}
+	*bup++ = '.';
+	*bup++ = '.';
+	*bup = '\0';
 
-        /* Open and stat parent directory. */
-        if (!(dir = opendir(up)) || fstat(dir->dd_fd, &s))
-            goto err;
+	/* Open and stat parent directory. */
+	if (!(dir = opendir(up)) || fstat(dir->dd_fd, &s))
+	    goto err;
 
-        /* Add trailing slash for next directory. */
-        *bup++ = '/';
+	/* Add trailing slash for next directory. */
+	*bup++ = '/';
 
-        /*
-         * If it's a mount point, have to stat each element because
-         * the inode number in the directory is for the entry in the
-         * parent directory, not the inode number of the mounted file.
-         */
-        save_errno = 0;
-        if (s.st_dev == dev) {
-            for (;;) {
-                if (!(dp = readdir(dir)))
-                    goto notfound;
-                if (dp->d_fileno == ino)
-                    break;
-            }
-        } else
-            for (;;) {
-                if (!(dp = readdir(dir)))
-                    goto notfound;
-                if (ISDOT(dp))
-                    continue;
-                bcopy(dp->d_name, bup, DNAMLEN(dp) + 1);
+	/*
+	 * If it's a mount point, have to stat each element because
+	 * the inode number in the directory is for the entry in the
+	 * parent directory, not the inode number of the mounted file.
+	 */
+	save_errno = 0;
+	if (s.st_dev == dev) {
+	    for (;;) {
+		if (!(dp = readdir(dir)))
+		    goto notfound;
+		if (dp->d_fileno == ino)
+		    break;
+	    }
+	}
+	else
+	    for (;;) {
+		if (!(dp = readdir(dir)))
+		    goto notfound;
+		if (ISDOT(dp))
+		    continue;
+		bcopy(dp->d_name, bup, DNAMLEN(dp) + 1);
 
-                /* Save the first error for later. */
-                if (lstat(up, &s)) {
-                    if (!save_errno)
-                        save_errno = errno;
-                    errno = 0;
-                    continue;
-                }
-                if (s.st_dev == dev && s.st_ino == ino)
-                    break;
-            }
+		/* Save the first error for later. */
+		if (lstat(up, &s)) {
+		    if (!save_errno)
+			save_errno = errno;
+		    errno = 0;
+		    continue;
+		}
+		if (s.st_dev == dev && s.st_ino == ino)
+		    break;
+	    }
 
-        /*
-         * Check for length of the current name, preceding slash,
-         * leading slash.
-         */
-        if (bpt - pt <= DNAMLEN(dp) + (first ? 1 : 2)) {
-            size_t len, off;
+	/*
+	 * Check for length of the current name, preceding slash,
+	 * leading slash.
+	 */
+	if (bpt - pt <= DNAMLEN(dp) + (first ? 1 : 2)) {
+	    size_t len, off;
 
-            if (!ptsize) {
-                errno = ERANGE;
-                goto err;
-            }
-            off = bpt - pt;
-            len = ept - bpt;
-            if (!(ptr = (char *)realloc(pt, ptsize *= 2)))
-                goto err;
-            pt = ptr;
-            bpt = pt + off;
-            ept = pt + ptsize;
-            (void)bcopy(bpt, ept - len, len);
-            bpt = ept - len;
-        }
-        if (!first)
-            *--bpt = '/';
-        bpt -= DNAMLEN(dp);
-        bcopy(dp->d_name, bpt, DNAMLEN(dp));
-        (void)closedir(dir);
+	    if (!ptsize) {
+		errno = ERANGE;
+		goto err;
+	    }
+	    off = bpt - pt;
+	    len = ept - bpt;
+	    if (!(ptr = (char *) realloc(pt, ptsize *= 2)))
+		goto err;
+	    pt = ptr;
+	    bpt = pt + off;
+	    ept = pt + ptsize;
+	    (void) bcopy(bpt, ept - len, len);
+	    bpt = ept - len;
+	}
+	if (!first)
+	    *--bpt = '/';
+	bpt -= DNAMLEN(dp);
+	bcopy(dp->d_name, bpt, DNAMLEN(dp));
+	(void) closedir(dir);
 
-        /* Truncate any file name. */
-        *bup = '\0';
+	/* Truncate any file name. */
+	*bup = '\0';
     }
 
-notfound:
+  notfound:
     /*
      * If readdir set errno, use it, not any saved error; otherwise,
      * didn't find the current directory in its parent directory, set
      * errno to ENOENT.
      */
     if (!errno)
-        errno = save_errno ? save_errno : ENOENT;
+	errno = save_errno ? save_errno : ENOENT;
     /* FALLTHROUGH */
-err:
+  err:
     if (ptsize && !pt)
-        free(pt);
+	free(pt);
     if (!up)
-        free(up);
-    return((char *)NULL);
+	free(up);
+    return ((char *) NULL);
 }
 
 char *
-getwd(b)
-    char *b;
+     getwd(b)
+     char *b;
 {
     char *p;
     uid_t euid = geteuid();
@@ -629,7 +625,7 @@ getwd(b)
     getcwd(b, MAXPATHLEN);
     p = getcwd(b, MAXPATHLEN);
     seteuid(euid);
-    return(p);
+    return (p);
 }
 
 /*
@@ -665,27 +661,27 @@ getwd(b)
  * SUCH DAMAGE.
  */
 
-# if !defined(lint)
+#if !defined(lint)
 static char sccsid[] = "@(#)initgroups.c    5.6 (Berkeley) 6/1/90";
-static char  rcsid[] = "@(#)$Id: initgroups.c,v 5.6.0.5 1992/02/23 06:56:52 paul Exp $";
-# endif /* !lint */
+static char rcsid[] = "@(#)$Id: sco.c,v 1.6 1999/08/27 14:18:42 wuftpd Exp $";
+#endif /* !lint */
 
 #include <stdio.h>
 
-# include <grp.h>
-# ifndef NGROUPS
-#  ifdef NGROUPS_MAX
-#   define NGROUPS  NGROUPS_MAX
-#  else /* !NGROUPS_MAX */
-#   define NGROUPS  8
-#  endif /* NGROUPS_MAX */
-# endif /* !NGROUPS */
+#include <grp.h>
+#ifndef NGROUPS
+#ifdef NGROUPS_MAX
+#define NGROUPS  NGROUPS_MAX
+#else /* !NGROUPS_MAX */
+#define NGROUPS  8
+#endif /* NGROUPS_MAX */
+#endif /* !NGROUPS */
 
 struct group *getgrent();
 
 initgroups(uname, agroup)
-    char *uname;
-    int agroup;
+     char *uname;
+     int agroup;
 {
     int groups[NGROUPS], ngroups = 0;
     register struct group *grp;
@@ -697,27 +693,34 @@ initgroups(uname, agroup)
      * and will be overwritten when a setgid file is executed.
      */
     if (agroup >= 0) {
-        groups[ngroups++] = agroup;
-        groups[ngroups++] = agroup;
+	groups[ngroups++] = agroup;
+	groups[ngroups++] = agroup;
     }
     setgrent();
     while (grp = getgrent()) {
-        if (grp->gr_gid == agroup)
-            continue;
-        for (i = 0; grp->gr_mem[i]; i++)
-            if (!strcmp(grp->gr_mem[i], uname)) {
-                if (ngroups == NGROUPS) {
-fprintf(stderr, "initgroups: %s is in too many groups\n", uname);
-                    goto toomany;
-                }
-                groups[ngroups++] = grp->gr_gid;
-            }
+	if (grp->gr_gid == agroup)
+	    continue;
+	for (i = 0; grp->gr_mem[i]; i++)
+	    if (!strcmp(grp->gr_mem[i], uname)) {
+		if (ngroups == NGROUPS) {
+		    fprintf(stderr, "initgroups: %s is in too many groups\n", uname);
+		    goto toomany;
+		}
+		groups[ngroups++] = grp->gr_gid;
+	    }
     }
-toomany:
+  toomany:
     endgrent();
     if (setgroups(ngroups, groups) < 0) {
-        perror("setgroups");
-        return (-1);
+	perror("setgroups");
+	return (-1);
     }
     return (0);
 }
+#else
+/* this will keep the preprocessor quiet */
+int SCOdummy()
+{
+    return (0);
+}
+#endif /* _SCO_DS */
